@@ -212,6 +212,17 @@ class ETreeTestCaseBase(unittest.TestCase):
         self.assertEquals('<a>&lt;&gt;&amp;</a>',
                          tostring(a))
 
+    def test_text_str_subclass(self):
+        Element = self.etree.Element
+
+        class strTest(str):
+            pass
+
+        a = Element("a")
+        a.text = strTest("text")
+        self.assertXML('<a>text</a>',
+                       a)
+
     def test_tail(self):
         ElementTree = self.etree.ElementTree
         
@@ -224,6 +235,18 @@ class ETreeTestCaseBase(unittest.TestCase):
         self.assertEquals('mixed', root[0].text)
         self.assertEquals(' content.', root[0].tail)
 
+    def test_tail_str_subclass(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        class strTest(str):
+            pass
+
+        a = Element("a")
+        SubElement(a, "t").tail = strTest("tail")
+        self.assertXML('<a><t></t>tail</a>',
+                       a)
+
     def test_ElementTree(self):
         Element = self.etree.Element
         ElementTree = self.etree.ElementTree
@@ -368,6 +391,13 @@ class ETreeTestCaseBase(unittest.TestCase):
             True,
             '{http://ns.codespeak.net/test}baz' in root.attrib)
 
+    def test_attribute_set(self):
+        Element = self.etree.Element
+
+        root = Element("root")
+        root.set("attr", "TEST")
+        self.assertEquals("TEST", root.get("attr"))
+
     def test_XML(self):
         XML = self.etree.XML
         
@@ -888,6 +918,17 @@ class ETreeTestCaseBase(unittest.TestCase):
         self.assertEquals('c',  a.tag)
         self.assertEquals('<c', tostring(a)[:2])
 
+    def test_tag_str_subclass(self):
+        Element = self.etree.Element
+
+        class strTest(str):
+            pass
+
+        a = Element("a")
+        a.tag = strTest("TAG")
+        self.assertXML('<TAG></TAG>',
+                       a)
+
     def test_delitem(self):
         Element = self.etree.Element
         SubElement = self.etree.SubElement
@@ -1154,6 +1195,7 @@ class ETreeTestCaseBase(unittest.TestCase):
         self.assertXML(
             '<a></a>',
             a)
+        self.assertEquals('b2', b.tail)
         
     def test_getchildren(self):
         Element = self.etree.Element
@@ -1938,6 +1980,20 @@ class ETreeTestCaseBase(unittest.TestCase):
         el = self.etree.parse(StringIO(isoxml)).getroot()
         self.assertEquals(utext, el.text)
 
+    def test_deepcopy_elementtree(self):
+        Element = self.etree.Element
+        ElementTree = self.etree.ElementTree
+
+        a = Element('a')
+        a.text = "Foo"
+        atree = ElementTree(a)
+
+        btree = copy.deepcopy(atree)
+        self.assertEqual("Foo", atree.getroot().text)
+        self.assertEqual("Foo", btree.getroot().text)
+        self.assertFalse(btree is atree)
+        self.assertFalse(btree.getroot() is atree.getroot())
+
     def test_deepcopy(self):
         Element = self.etree.Element
         
@@ -2004,20 +2060,6 @@ class ETreeTestCaseBase(unittest.TestCase):
         self.assertEquals(
             root[0][0].get('{tns}foo'),
             copy.deepcopy(root[0][0]).get('{tns}foo') )
-
-    def test_shallowcopy(self):
-        Element = self.etree.Element
-        
-        a = Element('a')
-        a.text = 'Foo'
-
-        b = copy.copy(a)
-        self.assertEquals('Foo', b.text)
-        
-        b.text = 'Bar'
-        self.assertEquals('Bar', b.text)
-        self.assertEquals('Foo', a.text)
-        # XXX ElementTree will share nodes, but lxml.etree won't..
         
     def test_deepcopy_append(self):
         # previously caused a crash
@@ -2034,6 +2076,33 @@ class ETreeTestCaseBase(unittest.TestCase):
         self.assertEquals('<a><X/></a>',
                           tostring(b).replace(' ', ''))
 
+    def test_shallowcopy(self):
+        Element = self.etree.Element
+        
+        a = Element('a')
+        a.text = 'Foo'
+
+        b = copy.copy(a)
+        self.assertEquals('Foo', b.text)
+        
+        b.text = 'Bar'
+        self.assertEquals('Bar', b.text)
+        self.assertEquals('Foo', a.text)
+        # XXX ElementTree will share nodes, but lxml.etree won't..
+
+    def test_shallowcopy_elementtree(self):
+        Element = self.etree.Element
+        ElementTree = self.etree.ElementTree
+        
+        a = Element('a')
+        a.text = 'Foo'
+        atree = ElementTree(a)
+
+        btree = copy.copy(atree)
+        self.assertFalse(btree is atree)
+        self.assert_(btree.getroot() is atree.getroot())
+        self.assertEquals('Foo', atree.getroot().text)
+
     def test_element_boolean(self):
         etree = self.etree
         e = etree.Element('foo')
